﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using Steema.TeeChart.WPF.Functions;
using Steema.TeeChart.WPF.Styles;
using TeeChartTicket.Annotations;
using Line = Steema.TeeChart.WPF.Styles.Line;

namespace TeeChartTicket
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private Line _series1 = null;

        readonly Model _model = null;
        public MainWindow()
        {
            _model = new Model();
            _model.LoadData();
            DataContext = _model;
            InitializeComponent();
            InitializeChart();
        }

        private void InitializeChart()
        {

            tChart1.Axes.Bottom.Automatic = true;
            tChart1.Axes.Bottom.Labels.ValueFormat = string.Empty;
            tChart1.Axes.Bottom.Labels.DateTimeFormat = string.Format("MM/dd/yyyy {0} HH:mm",Environment.NewLine);
            tChart1.Aspect.View3D = false;
            tChart1.Legend.Visible = false;
            tChart1.Aspect.ClipPoints = false;
            tChart1.Zoom.Allow = true;
            tChart1.Axes.Left.Automatic = true;

           
            //create new line
            _series1 = new Line(tChart1.Chart);
            _series1.Marks.Callout.ArrowHead = ArrowHeadStyles.Solid;
            _series1.Marks.Callout.ArrowHeadSize = 8;
            _series1.XValues.DateTime = true;
            _series1.LinePen.Width = 1;
            _series1.DefaultNullValue = double.NegativeInfinity;
            _series1.TreatNulls = TreatNullsStyle.DoNotPaint;

            _series1.Pointer.Visible = false;
            _series1.Pointer.Style = PointerStyles.DownTriangle;
            _series1.Pointer.Color = Colors.Red;
            _series1.Pointer.HorizSize = 4;
            _series1.Pointer.VertSize = 4;
            _series1.Pointer.Pen.Visible = false;

            _series1.GetPointerStyle += new CustomPoint.GetPointerStyleEventHandler(series1_GetPointerStyle);


            //does down sampling work with Line ?
            var downSampling1 = new DownSampling(tChart1.Chart) { Method = DownSamplingMethod.MinMaxFirstLast };
            _series1.Function = downSampling1;
            _series1.ColorEach = true;
            foreach (var variableValue in _model.Data)
            {
                if (variableValue.Value == null || variableValue.Value == double.NaN)
                    continue;
                _series1.Add(variableValue.ValueDateTime, variableValue.Value, "",
                            variableValue.Quality == Quality.Bad ? Colors.Black : _model.SelectedColor.Color);
            }
        }

        void series1_GetPointerStyle(CustomPoint series, GetPointerStyleEventArgs e)
        {
            Line dqSeries = series as Line;

            if (dqSeries != null)
            {
                //need pointer at start and end of "Bad Qulity" segment and "Segment with null values"
                //
            }
        }

        private VariableValues _rowBeingEdited = null;
        /// <summary>
        /// Handles the OnCellEditEnding event of the DataGrid control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Controls.DataGridCellEditEndingEventArgs"/> instance containing the event data.</param>
        private void DataGrid_OnCellEditEnding(object sender, DataGridCellEditEndingEventArgs e)
        {
            if (e.EditAction == DataGridEditAction.Commit)
            {

                _rowBeingEdited = e.Row.Item as VariableValues;
          
            }
        }

        /// <summary>
        /// Handles the OnCurrentCellChanged event of the DataGrid control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void DataGrid_OnCurrentCellChanged(object sender, EventArgs e)
        {
            if (_rowBeingEdited != null && datagrid.CurrentCell.Item != _rowBeingEdited)
            {
                datagrid.CommitEdit(DataGridEditingUnit.Row, true);
                _series1.Function.Recalculate();
            }
        }




        /// <summary>
        /// Handles the OnDropDownClosed event of the ComboBox control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void ComboBox_OnDropDownClosed(object sender, EventArgs e)
        {
            _series1.Clear();
            foreach (var variableValue in _model.Data)
            {
                _series1.Add(variableValue.ValueDateTime, variableValue.Value, "",
                            variableValue.Quality == Quality.Bad ? Colors.Black : _model.SelectedColor.Color);
            }
        }
    }

    /// <summary>
    /// 
    /// </summary>
    public class Model : INotifyPropertyChanged
    {
        /// <summary>
        /// Gets or sets the colors list.
        /// </summary>
        /// <value>
        /// The colors list.
        /// </value>
        public List<SolidColorBrush> ColorsList { get; set; }

        /// <summary>
        /// Gets or sets the color of the selected.
        /// </summary>
        /// <value>
        /// The color of the selected.
        /// </value>
        public SolidColorBrush SelectedColor { get; set; }

        public Model()
        {
            ColorsList = new List<SolidColorBrush>()
              {
                  new SolidColorBrush(Colors.Red),
                  new SolidColorBrush(Colors.Green),
                  new SolidColorBrush(Colors.GreenYellow),
                  new SolidColorBrush(Colors.Gold),
                  new SolidColorBrush(Colors.MediumBlue)
              };
            SelectedColor = ColorsList.First();
            _data = new ObservableCollection<VariableValues>();
        }
        /// <summary>
        /// Loads the data.
        /// </summary>
        public void LoadData()
        {
            String[] values = File.ReadAllText(@"Data.csv").Split(new string[] { Environment.NewLine }, StringSplitOptions.None);
            DateTime sTime = DateTime.Now;
            for (int index  = 0; index < values.Length; index++)
            {

                string value = values[index];
                if((index > 200 & index < 500) 
                    || (index > 3000 & index < 5000) )
                    Data.Add(new VariableValues { Value = Double.Parse(value), ValueDateTime = sTime, Quality = Quality.Bad });
                else if (index > 1000 & index < 2000)
                    Data.Add(new VariableValues { Value = null, ValueDateTime = sTime, Quality = Quality.Good });
                else
                    Data.Add(new VariableValues {Value = Double.Parse(value), ValueDateTime = sTime,Quality = Quality.Good});
                sTime = sTime.AddMinutes(1);
            }
        }

        private ObservableCollection<VariableValues> _data;
        /// <summary>
        /// Gets or sets the data.
        /// </summary>
        /// <value>
        /// The data.
        /// </value>
        public ObservableCollection<VariableValues> Data
        {
            get { return _data; }
            set
            {
                if (null != value)
                {
                    _data = value;
                    OnPropertyChanged("Data");
                }
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        [NotifyPropertyChangedInvocator]
        protected virtual void OnPropertyChanged(string propertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (handler != null) handler(this, new PropertyChangedEventArgs(propertyName));
        }
    }
    /// <summary>
    /// 
    /// </summary>
    public class VariableValues
    {
        /// <summary>
        /// Gets or sets the value date time.
        /// </summary>
        /// <value>
        /// The value date time.
        /// </value>
        public DateTime ValueDateTime { get; set; }
        /// <summary>
        /// Gets or sets the value.
        /// </summary>
        /// <value>
        /// The value.
        /// </value>
        public double? Value { get; set; }
        /// <summary>
        /// Gets or sets the quality.
        /// </summary>
        /// <value>
        /// The quality.
        /// </value>
        public Quality Quality { get; set; }
    }

    /// <summary>
    /// 
    /// </summary>
    public enum Quality
    {
        /// <summary>
        /// 
        /// </summary>
        Good,
        /// <summary>
        /// 
        /// </summary>
        Bad
    }
 
}
